function [ U ] = pdecol( varargin )
% PDECOL - Solve a PDE with algorithm 540 (collocation method).
% 
% Written by:
% -- 
% John L. Weatherwax                2005-04-27
% 
% email: wax@alum.mit.edu
% 
% Please send comments and especially bug reports to the
% above email address.
% 
%-----

% Ideas on how to inprove, passing in the number of pde's to solve?
npde                        = 2;   
%npde                        = 1;   

% Default pdecol arguments: 
%  
t0                          = 0.0;  
xL                          = 0; 
xR                          = 1; 
%nint                        = 30;
nint                        = 100;
dt                          = 1e-7;
eps                         = 1e-4; 
kord                        = 4;
ncc                         = 2; 
% Collocation points (not necessarily uniformly spaced):
xbkpt                       = linspace( xL, xR, nint+1 ); 
% x locations of the desired solution:
%xSol = ; 

% Times the solution is desired:
tGrid                       = linspace( t0, t0+1, 10 ); 
%tGrid                       = [ t0 1d-3 ];
%
% MF = 10*METH + MITER
% 
% METH=1 (ADAMS method (a generilization of crank-nicolson)
%     =2 (BACKWARDS DIFFERENTIATION FORMULAS (BDF) OR STIFF METHOD OF GEAR)
% MITER=1 (CHORD METHOD WITH ANALYTIC JACOBIANS, USER SUPPLIES SUBROUTINE DERIVF
%      =2 CHORD METHOD CALCULATED WITH FINITE DIFFERENCES
%
mf                          = 21; 
%mf                          = 12; 

% Number of derivatives to compute:
nderv                       = 0; 

% Override default arguments with "varargin" inputs:
%
i=1; 
while i <= nargin
  arg = varargin{i}
  switch lower(arg)
   case 'xl'
    xL = varargin{i+1};
   case 'xr'
    xR = varargin{i+1};
   case 'epsilon' 
    eps = varargin{i+1};
   case 'kord'
    kord = varargin{i+1};
   case 'ncc'
    ncc = varargin{i+1};
   case 'xbkpt'
    xbkpt = varargin{i+1};
   case 'tGrid'
    tGrid = varargin{i+1};
   case 'mf'
    mf = varargin{i+1};
   otherwise
    error( [ 'Unknown input', arg ] ); 
  end
  i=i+2;
end

% Error checking:
%
if( xL >= xR ) 
  error( 'xL >= xR' ); 
end
if( eps <= 0 )
  error( 'eps <= 0' ); 
end
if( kord < 3 | kord > 20 )
  error( 'kord < 3 | kord > 20' ); 
end
if( ncc <= 1 | ncc > kord )
  error( 'ncc <= 1 | ncc > kord' ); 
end
if( nint < 1 )
  error( 'nint < 1' ); 
end
if( length(tGrid) < 2 ) 
  error( 'length(tGrid) < 2' ); 
end
if( nderv >= kord ) 
  error( 'nderv >= kord' ); 
end

% Calculate the required data arrays sizes:
%
nbkpt       = nint+1;
ncpts       = kord*nint - ncc*(nint-1);
iquad       = 1;                          % assume the larger value (v.s. 0)
ml          = npde*(kord+iquad-1) - 1;
maxder      = 5;

nwork       = kord + 4*npde + 9*npde^2 + ncpts*(3*kord + 2) + ...
                                    npde*ncpts*(3*ml + maxder + 7);
niwork      = ncpts*(npde+1);
nsctch      = kord*(nderv+1);

% Call a octave *.oct file that calls the FORTRAN pdecol.f:
%
[U] = pdecolg(tGrid,dt,xbkpt,eps,kord,ncc,mf,npde,nwork,niwork,nsctch);

% Plot the solution U:
ntouts = length(tGrid)-1; 
%h=figure; 
for nti=1:ntouts,
  %figure(h); 
  figure(nti); 
  subplot( 1,2,1 );
  plot( xbkpt, U(1,:,nti), '-o' ); grid on; %title( [ 'U(1) at t(',num2str(nti),')' ] ); 
  %pause; 
  subplot( 1,2,2 ); 
  plot( xbkpt, U(2,:,nti), '-o' ); grid on; %title( [ 'U(2) at t(',num2str(nti),')' ] ); 
  %pause;
end

% Collect all inputs together into a structure:
%
opts.npde           = npde;

opts.t0             = t0;
opts.xL             = xL;
opts.xR             = xR; 
opts.nint           = nint; 
opts.dt             = dt;
opts.eps            = eps;
opts.kord           = kord;
opts.ncc            = ncc;
opts.xbkpt          = xbkpt;
opts.tGrid          = tGrid;
opts.mf             = mf;

opts.nderv          = nderv;
